#type vertex
#version 450 core

layout(location = 0) in float a_ScreenSpace;
layout(location = 1) in vec4 a_Position;
layout(location = 2) in vec4 a_Color;
layout(location = 3) in vec2 a_TexCoord;
layout(location = 4) in float a_TextureTilingFactor;
layout(location = 5) in vec3 a_Normal;

layout(std140, binding = 0) uniform Camera { mat4 u_ViewProjection; };
layout(std140, binding = 1) uniform Color { vec4 u_tintColor;
mat4 u_transform; };

struct VertexOutput {
  vec4 Position;
  vec4 Color;
  vec2 TexCoord;
  float TextureTilingFactor;
  vec3 Normal;
};

layout(location = 0) out VertexOutput Output;

void main() {
  vec4 newTransform = u_ViewProjection * u_transform * a_Position;
  Output.Position = newTransform;
  Output.Color = a_Color * u_tintColor;
  Output.TexCoord = a_TexCoord;
  Output.TextureTilingFactor = 1;
  Output.Normal = a_Normal;
  
  gl_Position = newTransform;
}

#type fragment
#version 450 core

layout(location = 0) out vec4 color;

struct VertexOutput {
  vec4 Position;
  vec4 Color;
  vec2 TexCoord;
  float TextureTilingFactor;
  vec3 Normal;
};

layout(location = 0) in VertexOutput Input;

layout(binding = 0) uniform sampler2D u_Texture;

void main() {

  float DiffusedLightIntensity = 7.0f; // Sets intensity of Diffused Light
  vec3 LightColor = vec3(0.1f, 0.1f, 0.1f); // Sets the color of ambient and diffused light
  vec3 lightPosition = vec3(3, 4, 0); // Sets position of diffused light source

  vec3 norm = normalize(Input.Normal);
  vec3 lightDirection = normalize(lightPosition - Input.Position.xyz);

  float diffusedIntensity = max(dot(norm, lightDirection), 0.0);
  vec3 diffusedLight = diffusedIntensity * (DiffusedLightIntensity*LightColor);
  vec4 diffusedLightAlpha = vec4(diffusedLight, 1.0f); // Sets alpha of diffused light

  vec4 AmbientColor = vec4(LightColor, 1.0f); // Sets alpha of ambient light

  color = Input.Color * (AmbientColor + diffusedLightAlpha) * texture(u_Texture, Input.TexCoord * Input.TextureTilingFactor); 

  if(color.a < 0.5)
    discard;

}
